==> apache-ignite-2.16.0-src.zip: apache-ignite-2.16.0-src/modules/calcite/src/main/java/org/apache/ignite/internal/processors/query/calcite/schema/IgniteIndex.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.ignite.internal.processors.query.calcite.schema;

import java.util.List;
import org.apache.calcite.plan.RelOptCluster;
import org.apache.calcite.plan.RelOptTable;
import org.apache.calcite.rel.RelCollation;
import org.apache.calcite.rex.RexNode;
import org.apache.calcite.util.ImmutableBitSet;
import org.apache.ignite.internal.processors.query.calcite.exec.ExecutionContext;
import org.apache.ignite.internal.processors.query.calcite.exec.exp.RangeIterable;
import org.apache.ignite.internal.processors.query.calcite.metadata.ColocationGroup;
import org.apache.ignite.internal.processors.query.calcite.prepare.bounds.SearchBounds;
import org.apache.ignite.internal.processors.query.calcite.rel.logical.IgniteLogicalIndexScan;
import org.jetbrains.annotations.Nullable;

/**
 * Ignite scannable index.
 */
public interface IgniteIndex {
    /** */
    public RelCollation collation();

    /** */
    public String name();

    /** */
    public IgniteTable table();

    /**
     * Converts index into relational expression.
     *
     * @param cluster         Custer.
     * @param relOptTbl       Table.
     * @param proj            List of required projections.
     * @param cond            Conditions to filter rows.
     * @param requiredColumns Set of columns to extract from original row.
     * @return Table relational expression.
     */
    public IgniteLogicalIndexScan toRel(
        RelOptCluster cluster,
        RelOptTable relOptTbl,
        @Nullable List<RexNode> proj,
        @Nullable RexNode cond,
        @Nullable ImmutableBitSet requiredColumns
    );

    /**
     * Converts condition to index find predicate.
     *
     * @param cluster         Custer.
     * @param cond            Conditions to filter rows.
     * @param requiredColumns Set of columns to extract from original row.
     * @return Index condition.
     */
    public List<SearchBounds> toSearchBounds(
        RelOptCluster cluster,
        @Nullable RexNode cond,
        @Nullable ImmutableBitSet requiredColumns
    );

    /** */
    public <Row> Iterable<Row> scan(
        ExecutionContext<Row> execCtx,
        ColocationGroup grp,
        RangeIterable<Row> ranges,
        @Nullable ImmutableBitSet requiredColumns
    );

    /**
     * Calculates index records number.
     *
     * @param ectx Execution context.
     * @param grp  Colocation group.
     * @param notNull Exclude null values.
     * @return Index records number for {@code group}.
     */
    public long count(ExecutionContext<?> ectx, ColocationGroup grp, boolean notNull);

    /**
     * Takes only first or last not-null index value.
     *
     * @param first {@code True} to take first index not-null value. {@code False} for last.
     * @param ectx Execution context.
     * @param grp Colocation group.
     * @param requiredColumns  Required columns.
     * @return Index records for {@code grp}.
     */
    public <Row> Iterable<Row> firstOrLast(
        boolean first,
        ExecutionContext<Row> ectx,
        ColocationGroup grp,
        @Nullable ImmutableBitSet requiredColumns
    );

    /**
     * If its possible to scan requred columns using inlined index keys.
     */
    public boolean isInlineScanPossible(@Nullable ImmutableBitSet requiredColumns);
}

==> apache-ignite-2.17.0-src.zip: apache-ignite-2.17.0-src/modules/calcite/src/main/java/org/apache/ignite/internal/processors/query/calcite/schema/IgniteIndex.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.ignite.internal.processors.query.calcite.schema;

import java.util.List;
import org.apache.calcite.plan.RelOptCluster;
import org.apache.calcite.plan.RelOptTable;
import org.apache.calcite.rel.RelCollation;
import org.apache.calcite.rex.RexNode;
import org.apache.calcite.util.ImmutableBitSet;
import org.apache.ignite.internal.processors.query.calcite.exec.ExecutionContext;
import org.apache.ignite.internal.processors.query.calcite.exec.exp.RangeIterable;
import org.apache.ignite.internal.processors.query.calcite.metadata.ColocationGroup;
import org.apache.ignite.internal.processors.query.calcite.prepare.bounds.SearchBounds;
import org.apache.ignite.internal.processors.query.calcite.rel.logical.IgniteLogicalIndexScan;
import org.jetbrains.annotations.Nullable;

/**
 * Ignite scannable index.
 */
public interface IgniteIndex {
    /** */
    public RelCollation collation();

    /** */
    public String name();

    /** */
    public IgniteTable table();

    /**
     * Converts index into relational expression.
     *
     * @param cluster         Custer.
     * @param relOptTbl       Table.
     * @param proj            List of required projections.
     * @param cond            Conditions to filter rows.
     * @param requiredColumns Set of columns to extract from original row.
     * @return Table relational expression.
     */
    public IgniteLogicalIndexScan toRel(
        RelOptCluster cluster,
        RelOptTable relOptTbl,
        @Nullable List<RexNode> proj,
        @Nullable RexNode cond,
        @Nullable ImmutableBitSet requiredColumns
    );

    /**
     * Converts condition to index find predicate.
     *
     * @param cluster         Custer.
     * @param cond            Conditions to filter rows.
     * @param requiredColumns Set of columns to extract from original row.
     * @return Index condition.
     */
    public List<SearchBounds> toSearchBounds(
        RelOptCluster cluster,
        @Nullable RexNode cond,
        @Nullable ImmutableBitSet requiredColumns
    );

    /** */
    public <Row> Iterable<Row> scan(
        ExecutionContext<Row> execCtx,
        ColocationGroup grp,
        RangeIterable<Row> ranges,
        @Nullable ImmutableBitSet requiredColumns
    );

    /**
     * Calculates index records number.
     *
     * @param ectx Execution context.
     * @param grp  Colocation group.
     * @param notNull Exclude null values.
     * @return Iterable with one row and one col: records count by index for {@code group}.
     */
    public <Row> Iterable<Row> count(ExecutionContext<Row> ectx, ColocationGroup grp, boolean notNull);

    /**
     * Takes only first or last not-null index value.
     *
     * @param first {@code True} to take first index not-null value. {@code False} for last.
     * @param ectx Execution context.
     * @param grp Colocation group.
     * @param requiredColumns  Required columns.
     * @return Index records for {@code grp}.
     */
    public <Row> Iterable<Row> firstOrLast(
        boolean first,
        ExecutionContext<Row> ectx,
        ColocationGroup grp,
        @Nullable ImmutableBitSet requiredColumns
    );

    /**
     * If its possible to scan requred columns using inlined index keys.
     */
    public boolean isInlineScanPossible(@Nullable ImmutableBitSet requiredColumns);
}
